@@ -496,7 +496,7 @@ class DebugClientBase(object):
                 wd, fn, args, tracePython = arg.split('|')
                 self.__setCoding(fn)
                 sys.argv.append(fn)
-                sys.argv.extend(eval(args.replace("u'", "'")))
+                sys.argv.extend(eval(args))
                 sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
@@ -542,7 +542,7 @@ class DebugClientBase(object):
                 wd, fn, args = arg.split('|')
                 self.__setCoding(fn)
                 sys.argv.append(fn)
-                sys.argv.extend(eval(args.replace("u'", "'")))
+                sys.argv.extend(eval(args))
                 sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
@@ -575,7 +575,7 @@ class DebugClientBase(object):
                         res = exc.code
                         atexit._run_exitfuncs()
                     self.writestream.flush()
-                    self.progTerminated(res, exit=True)
+                    self.progTerminated(res)
                 return
 
             if cmd == DebugProtocol.RequestProfile:
@@ -585,7 +585,7 @@ class DebugClientBase(object):
                 wd, fn, args, erase = arg.split('|')
                 self.__setCoding(fn)
                 sys.argv.append(fn)
-                sys.argv.extend(eval(args.replace("u'", "'")))
+                sys.argv.extend(eval(args))
                 sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
@@ -621,7 +621,7 @@ class DebugClientBase(object):
                         atexit._run_exitfuncs()
                     self.prof.save()
                     self.writestream.flush()
-                    self.progTerminated(res, exit=True)
+                    self.progTerminated(res)
                 return
 
             if cmd == DebugProtocol.RequestCoverage:
@@ -630,7 +630,7 @@ class DebugClientBase(object):
                 wd, fn, args, erase = arg.split('@@')
                 self.__setCoding(fn)
                 sys.argv.append(fn)
-                sys.argv.extend(eval(args.replace("u'", "'")))
+                sys.argv.extend(eval(args))
                 sys.path = self.__getSysPath(os.path.dirname(sys.argv[0]))
                 if wd == '':
                     os.chdir(sys.path[1])
@@ -672,7 +672,7 @@ class DebugClientBase(object):
                     self.cover.stop()
                     self.cover.save()
                     self.writestream.flush()
-                    self.progTerminated(res, exit=True)
+                    self.progTerminated(res)
                 return
 
             if cmd == DebugProtocol.RequestShutdown:
@@ -775,7 +775,8 @@ class DebugClientBase(object):
                     value = eval(
                         arg,
                         self.currentThread.getCurrentFrame().f_globals,
-                        self.currentThread.getFrameLocals(0))
+                        self.currentThread.getFrameLocals(self.framenr))
+                    self.currentThread.storeFrameLocals(self.framenr)
                 except:
                     # Report the exception and the traceback
                     try:
@@ -806,10 +807,11 @@ class DebugClientBase(object):
             
             if cmd == DebugProtocol.RequestExec:
                 _globals = self.currentThread.getCurrentFrame().f_globals
-                _locals = self.currentThread.getFrameLocals(0)
+                _locals = self.currentThread.getFrameLocals(self.framenr)
                 try:
                     code = compile(arg + '\n', '<stdin>', 'single')
                     exec(code, _globals, _locals)
+                    self.currentThread.storeFrameLocals(self.framenr)
                 except:
                     # Report the exception and the traceback
                     try:
@@ -996,6 +998,8 @@ class DebugClientBase(object):
                             _locals["sys"].stdout = __stdout
                         else:
                             exec(code, _globals, _locals)
+                        
+                        self.currentThread.storeFrameLocals(self.framenr)
                 except SystemExit as exc:
                     self.progTerminated(exc.code)
                 except:
@@ -1046,7 +1050,7 @@ class DebugClientBase(object):
 
     def __interact(self):
         """
-        Private method to Interact with  the debugger.
+        Private method to interact with the debugger.
         """
         global DebugClientInstance
 
@@ -1070,6 +1074,9 @@ class DebugClientBase(object):
         while self.eventExit is None:
             wrdy = []
 
+            if self.writestream.nWriteErrors > self.writestream.maxtries:
+                break
+            
             if AsyncPendingWrite(self.writestream):
                 wrdy.append(self.writestream)
 
@@ -1303,13 +1310,12 @@ class DebugClientBase(object):
         """
         return self.running
 
-    def progTerminated(self, status, exit=False):
+    def progTerminated(self, status):
         """
         Public method to tell the debugger that the program has terminated.
         
         @param status return status
-        @param exit flag indicating to perform a sys.exit()
-        @type bool
+        @type int
         """
         if status is None:
             status = 0
@@ -1323,9 +1329,6 @@ class DebugClientBase(object):
             self.set_quit()
             self.running = None
             self.write('{0}{1:d}\n'.format(DebugProtocol.ResponseExit, status))
-            if exit:
-                self.writestream.close(True)
-                sys.exit(status)
         
         # reset coding
         self.__coding = self.defaultCoding
@@ -1359,12 +1362,11 @@ class DebugClientBase(object):
                 scope = -1
         elif scope:
             dict = f.f_globals
+        elif f.f_globals is f.f_locals:
+                scope = -1
         else:
             dict = f.f_locals
             
-            if f.f_globals is f.f_locals:
-                scope = -1
-        
         varlist = [scope]
         
         if scope != -1:
@@ -1404,11 +1406,10 @@ class DebugClientBase(object):
                 scope = -1
         elif scope:
             dict = f.f_globals
+        elif f.f_globals is f.f_locals:
+                scope = -1
         else:
             dict = f.f_locals
-            
-            if f.f_globals is f.f_locals:
-                scope = -1
         
         varlist = [scope, var]
         
@@ -1935,8 +1936,7 @@ class DebugClientBase(object):
         """
         completerDelims = ' \t\n`~!@#$%^&*()-=+[{]}\\|;:\'",<>/?'
         
-        completions = []
-        state = 0
+        completions = set()
         # find position of last delim character
         pos = -1
         while pos >= -len(text):
@@ -1948,20 +1948,38 @@ class DebugClientBase(object):
                 break
             pos -= 1
         
+        # Get local and global completions
         try:
-            comp = self.complete(text, state)
-        except:
-            comp = None
-        while comp is not None:
-            completions.append(comp)
-            state += 1
-            try:
-                comp = self.complete(text, state)
-            except:
-                comp = None
+            localdict = self.currentThread.getFrameLocals(self.framenr)
+            localCompleter = Completer(localdict).complete
+            self.__getCompletionList(text, localCompleter, completions)
+        except AttributeError:
+            pass
+        self.__getCompletionList(text, self.complete, completions)
         
         self.write("{0}{1}||{2}\n".format(DebugProtocol.ResponseCompletion,
-                                          str(completions), text))
+                                          str(list(completions)), text))
+
+    def __getCompletionList(self, text, completer, completions):
+        """
+        Private method to create a completions list.
+        
+        @param text text to complete (string)
+        @param completer completer methode
+        @param completions set where to add new completions strings (set)
+        """
+        state = 0
+        try:
+            comp = completer(text, state)
+        except Exception:
+            comp = None
+        while comp is not None:
+            completions.add(comp)
+            state += 1
+            try:
+                comp = completer(text, state)
+            except Exception:
+                comp = None
 
     def startDebugger(self, filename=None, host=None, port=None,
                       enableTrace=True, exceptions=True, tracePython=False,
